﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BlobManager.Properties;

using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.IO;

namespace BlobManager
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.txtStorageKey.Text = Settings.Default.StorageKey;
            this.txtStorageName.Text = Settings.Default.StorageName;
            this.txtBlobContainerName.Text = Settings.Default.BlobContainerName;

            if (string.IsNullOrEmpty(this.txtStorageKey.Text) == false
                && string.IsNullOrEmpty(this.txtStorageName.Text) == false
                && string.IsNullOrEmpty(this.txtBlobContainerName.Text) == false)
            {
                this.btnRefresh_Click(null, EventArgs.Empty);
            }

            this.txtLocalPath.Text = Settings.Default.LocalPath;

            if (string.IsNullOrEmpty(this.txtLocalPath.Text) == false)
            {
                this.btnLocalView_Click(null, EventArgs.Empty);
            }

        }

        CloudStorageAccount _storageAccount;
        CloudBlobClient _blobClient;
        CloudBlobContainer _blobContainer;
        HashSet<string> _blobsHash;

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            Settings.Default.StorageKey = this.txtStorageKey.Text;
            Settings.Default.StorageName = this.txtStorageName.Text;
            Settings.Default.BlobContainerName = this.txtBlobContainerName.Text;
            Settings.Default.Save();

            string connectionString
            = string.Format("DefaultEndpointsProtocol=http;AccountName={0};AccountKey={1}",
            this.txtStorageName.Text, this.txtStorageKey.Text);

            _storageAccount = CloudStorageAccount.Parse(connectionString);
            _blobClient = _storageAccount.CreateCloudBlobClient();
            _blobContainer = _blobClient.GetContainerReference(this.txtBlobContainerName.Text);

            // 컨테이너 참조의 ListBlobs 메서드를 통해서 반환되는 블랍(BLOB)들의 Uri 확인합니다.
            lstRemote.Items.Clear();
            lstRemote.BeginUpdate();
            _blobsHash = new HashSet<string>();

            string localFirstFileName = string.Empty;

            if (this.lstLocal.SelectedItem != null)
            {
                localFirstFileName = Path.GetFileName(this.lstLocal.SelectedItem.ToString());
            }

            int firstIndex = -1;
            foreach (CloudBlockBlob blobItem in _blobContainer.ListBlobs())
            {
                lstRemote.Items.Add(blobItem);
                lstRemote.DisplayMember = "Name";
                _blobsHash.Add(blobItem.Name);

                if (firstIndex == -1)
                {
                    if (blobItem.Name.Equals(localFirstFileName) == true)
                    {
                        firstIndex = lstRemote.Items.Count - 1;
                    }
                }
            }
            lstRemote.EndUpdate();

            lstRemote.TopIndex = firstIndex;
        }

        private void btnLocalView_Click(object sender, EventArgs e)
        {
            Settings.Default.LocalPath = this.txtLocalPath.Text;
            Settings.Default.Save();

            this.lstLocal.Items.Clear();

            this.lstLocal.BeginUpdate();
            foreach (string filePath in Directory.GetFiles(this.txtLocalPath.Text))
            {
                this.lstLocal.Items.Add(filePath);
            }
            this.lstLocal.EndUpdate();
        }

        private void btnUpload_Click(object sender, EventArgs e)
        {
            foreach (string filePath in this.lstLocal.SelectedItems)
            {
                string fileName = Path.GetFileName(filePath);
                if (_blobsHash.Contains(fileName) == true)
                {
                    if (MessageBox.Show("Overwrite? " + fileName, "Conflicts", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.No)
                    {
                        continue;
                    }
                }

                CloudBlob aBlob = _blobContainer.GetBlobReference(fileName);
                using (var fileStream = System.IO.File.OpenRead(filePath))
                {
                    aBlob.UploadFromStream(fileStream);
                }
            }

            this.btnRefresh_Click(sender, e);
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            foreach (CloudBlockBlob blobItem in this.lstRemote.SelectedItems)
            {
                CloudBlob aBlob = _blobContainer.GetBlobReference(blobItem.Name);
                aBlob.Delete();
            }

            this.btnRefresh_Click(sender, e);
        }

    }
}
